<?php

namespace Drupal\paragraphs_blokkli_graphql\Plugin\GraphQL\DataProducer;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\graphql\GraphQL\Execution\FieldContext;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Performs the access check wether the current user can use blökkli.
 *
 * @DataProducer(
 *   id = "paragraphs_blokkli_can_use_blokkli",
 *   name = @Translation("Can use blökkli"),
 *   description = @Translation("Returns TRUE if the current user can use blökkli."),
 *   produces = @ContextDefinition("boolean",
 *     label = @Translation("TRUE if the current user can use blökkli.")
 *   ),
 * )
 */
class CanUseBlokkli extends DataProducerPluginBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_user'),
    );
  }

  /**
   * CurrentUser constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param array $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Session\AccountInterface $currentUser
   *   The current user.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    array $plugin_definition,
    protected AccountInterface $currentUser,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * Resolver.
   *
   * @param \Drupal\graphql\GraphQL\Execution\FieldContext $fieldContext
   *   The field context.
   *
   * @return bool
   *   TRUE if the current user can use blökkli.
   */
  public function resolve(FieldContext $fieldContext): bool {
    // In order to use blökkli, the user must have at least the permission to
    // edit an edit state.
    $hasPermission = $this->currentUser->hasPermission('edit paragraphs blokkli edit state');
    $fieldContext->addCacheContexts(['user.permissions']);
    return $hasPermission;
  }

}
